package net.wanji.opt.servicev2.implv2;

import lombok.extern.slf4j.Slf4j;
import net.wanji.opt.dao.mapper.TrunkLineMapper;
import net.wanji.opt.servicev2.TrunkLineService;
import net.wanji.opt.synthesis.pojo.TrunkLineCrossProblemEntity;
import net.wanji.opt.synthesis.pojo.TrunkLineProblemDescribeEntity;
import net.wanji.opt.vo2.CrossRealTimeAlarmVO;
import net.wanji.opt.vo2.TrunkLineCrossProblemVO;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

@Service
@Slf4j
public class TrunkLineImpl implements TrunkLineService {
    @Resource
    private TrunkLineMapper trunkLineMapper;

    @Override
    public List<TrunkLineProblemDescribeEntity> getTrunkLineProblemDescribe(String greenID , String time) {
        return trunkLineMapper.getTrunkLineProblemDescribe(greenID,time);
    }

    @Override
    public List<CrossRealTimeAlarmVO> getTrunkLineRealTimeAlarm(Integer greenID, String time) {
        return trunkLineMapper.getTrunkLineRealTimeAlarm(greenID, time);
    }

    @Override
    public List<TrunkLineCrossProblemVO> getTrunkLineCrossProblem(Integer greenID, String time) {
        List<TrunkLineCrossProblemEntity> list= trunkLineMapper.getTrunkLineCrossProblem(greenID, time);
        TrunkLineCrossProblemEntity ioDir = trunkLineMapper.getIODir(greenID);
        List<TrunkLineCrossProblemVO> voList = new ArrayList<>();

        for (TrunkLineCrossProblemEntity e : list) {
            TrunkLineCrossProblemVO vo = new TrunkLineCrossProblemVO();

            vo.setCrossID(e.getCrossID());
            vo.setName(e.getName());
            vo.setWkt(e.getWkt());

            if (e.getDirection() != null) {
                String temp = e.getDirection().substring(1, e.getDirection().length() - 1);
                temp = temp.replace("\"", "");
                String[] temp1 = temp.split(",");

                String[] dirStatus = getDirStatus(ioDir.getInDir(), ioDir.getOutDir(), temp1, e.getStatus());
                vo.setDir_1_name(dirStatus[0]);
                vo.setStatus_1_name(dirStatus[1]);
                vo.setDir_2_name(dirStatus[2]);
                vo.setStatus_2_name(dirStatus[3]);

                voList.add(vo);
            } else {
                Map<String, String> IOdirMap = getDirName(ioDir.getInDir(), ioDir.getOutDir());
                vo.setStatus_1_name("正常");
                vo.setStatus_2_name("正常");
                vo.setDir_1_name(IOdirMap.get("dirName"));
                vo.setDir_2_name(IOdirMap.get("dirNameReverse"));

                voList.add(vo);
            }
        }

        return voList;
    }

    private Map<String, String> getDirName(Integer inDir, Integer outDir) {
        Map<String, String> result = new HashMap<>();

        if (inDir == 5 && outDir == 1 || inDir == 1 && outDir == 5) { // 南向北
            result.put("dir", "6");
            result.put("dirReverse", "0");
            result.put("dirName", "南向北");
            result.put("dirNameReverse", "北向南");
            return result;
        } else if (inDir == 7 && outDir == 3 || inDir == 3 && outDir == 7) { // 西向东
            result.put("dir", "4");
            result.put("dirReverse", "2");
            result.put("dirName", "西向东");
            result.put("dirNameReverse", "东向西");
            return result;
        }
        return Collections.emptyMap();
    }

    private String getStatusName(Integer status) {
        switch(status) {
            case -1:
            case 0:
                return "正常";
            case 1:
                return "失衡";
            case 3:
                return "拥堵";
            case 4:
                return "死锁";
            case 5:
                return "相位空放";
        }
        return "";
    }

    /**
     * @param inDir  进口方向
     * @param outDir 出口方向
     * @param dirs   方向数组
     * @param status 状态
     * @return 方向状态数组[方向1, 状态1, 方向2, 状态2]
     */
    private String[] getDirStatus(Integer inDir, Integer outDir, String[] dirs, Integer status) {
        Map<String, String> IODirMap = getDirName(inDir, outDir);
        String[] result = new String[4];

        if (Arrays.asList(dirs).contains(IODirMap.get("dir"))
                && Arrays.asList(dirs).contains(IODirMap.get("dirReverse"))
        ) {
            result[0] = IODirMap.get("dirName");
            result[1] = getStatusName(status);
            result[2] = IODirMap.get("dirNameReverse");
            result[3] = getStatusName(status);
        } else if (Arrays.asList(dirs).contains(IODirMap.get("dirReverse"))
                && !Arrays.asList(dirs).contains(IODirMap.get("dir"))) {
            result[0] = IODirMap.get("dirName");
            result[1] = getStatusName(0);
            result[2] = IODirMap.get("dirNameReverse");
            result[3] = getStatusName(status);
        } else if (Arrays.asList(dirs).contains(IODirMap.get("dir"))
                && !Arrays.asList(dirs).contains(IODirMap.get("dirReverse"))) {
            result[0] = IODirMap.get("dirName");
            result[1] = getStatusName(status);
            result[2] = IODirMap.get("dirNameReverse");
            result[3] = getStatusName(0);
        } else {
            result[0] = IODirMap.get("dirName");
            result[1] = getStatusName(0);
            result[2] = IODirMap.get("dirNameReverse");
            result[3] = getStatusName(0);
        }

        return result;
    }

}
